/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2018 Intel Corporation. All rights reserved.
 *
 * Author: Adrian Bonislawski <adrian.bonislawski@linux.intel.com>
 */

/* Memory mock for memmap */
#define HEAP_SYSTEM_0_BASE	0xBE200000
#define HEAP_SYS_RUNTIME_0_BASE 0xBE220000
#define SOF_CORE_S_START	0xBE280000
#define HEAP_RUNTIME_SHARED_BASE	0xBE290000
#define HEAP_SYSTEM_SHARED_BASE		0xBE2B0000
#define HEAP_RUNTIME_BASE	0xBE2C0000
#define HEAP_BUFFER_BASE	0xBE3B0000
