/* SPDX-License-Identifier: BSD-3-Clause */
/*
 * Customer ID = 7578; Build = 0xa69ab; Copyright (c) 2020-2021 by Cadence Design Systems. Inc.
 * ALL RIGHTS RESERVED.
 * These coded instructions, statements, and computer programs are the
 * copyrighted works and confidential proprietary information of Tensilica Inc.
 * They may not be modified, copied, reproduced, distributed, or disclosed to
 * third parties in any manner, medium, or form, in whole or in part, without
 * the prior written consent of Cadence Design Systems Inc.
 */

#ifndef TENSILICA_CONFIG_KEY
#define TENSILICA_CONFIG_KEY 1

#include <xtensa/xtensa-types.h>

#endif

