function bf = sof_bf_defaults()

% Recording array general setup
bf.taylorwin_sidelobe = -30;
bf.chebwin_sidelobe = 30;
bf.taylorwin_nbar = 4;
bf.type_filt = 'hann'; % filter
bf.type = 'SDB';     % SDB for superdirectivem, DSB for delay and sum
bf.fs = 16e3;        % Design for 16 kHz sample rate
bf.c = 343;          % Speed of sound in 20C
bf.steer_az = 0;     % Azimuth 0 deg
bf.steer_el = 0;     % Elevation 0 deg
bf.steer_r = 5.0;    % Distance 5.0m
bf.fir_length = 64;  % 64 tap FIR filters
bf.kaiser_beta = 10;    % Beta for kaiser window method FIR design
bf.mu_db = -40;      % dB of diagonal loading to noise covariance matrix
bf.do_plots = 0;
bf.plot_box = 0.20;  % Show 20cm wide plot cube for array geometry
bf.array_angle = [0 0 0]; % Array rotation angles for xyz
bf.tplg_fn = '';
bf.sofctl_fn = '';
bf.ucmbin3_fn = '';
bf.ucmbin4_fn = '';
sof_tools = '../../../../tools';
bf.tplg1_path = fullfile(sof_tools, 'topology/topology1/m4/tdfb');
bf.tplg2_path = fullfile(sof_tools, 'topology/topology2/include/components/tdfb');
bf.sofctl3_path = fullfile(sof_tools, 'ctl/ipc3/tdfb');
bf.sofctl4_path = fullfile(sof_tools, 'ctl/ipc4/tdfb');
bf.data_path = './data';
bf.endian = 'little';
bf.fn = 1;
bf.create_simulation_data = false;
bf.sinerot_a = 10^(-20/20);
bf.sinerot_f = 2e3;
bf.sinerot_t = 1.0;
bf.sinerot_az_step = 5;
bf.sinerot_az_start = -180;
bf.sinerot_az_stop = 180;
bf.diffuse_t = 1;
bf.diffuse_lev = -20;
bf.random_t = 1;
bf.random_lev = -20;
bf.num_output_channels = 1;
bf.num_output_streams = 1;
bf.input_channel_select = [];
bf.output_channel_mix_beam_off = [];
bf.output_channel_mix = [];
bf.output_stream_mix = [];
bf.num_filters = [];
bf.angle_enum_mult = 30;
bf.angle_enum_offs = 0;
bf.track_doa = 0;
bf.beam_off_defined = 1;
bf.array = '';
bf.mic_x = [];
bf.mic_y = [];
bf.mic_z = [];
bf.mic_n = 0;
bf.array_id = {''};
bf.noiserot_fn = {''};
bf.sinerot_fn = {''};
bf.diffuse_fn = {''};
bf.random_fn = {''};
bf.mat_fn = {''};
bf.export_note = '';
bf.export_howto = '';
end
