/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2024 Google LLC.  All rights reserved.
 * Author: Andy Ross <andyross@google.com>
 */
#ifndef _SOF_PLATFORM_MTK_LIB_CPU_H
#define _SOF_PLATFORM_MTK_LIB_CPU_H

#define PLATFORM_PRIMARY_CORE_ID 0

#endif /* _SOF_PLATFORM_MTK_LIB_CPU_H */
