/* SPDX-License-Identifier: BSD-3-Clause */
/*
 * Copyright(c) 2021 MediaTek. All rights reserved.
 *
 * Author: Bo Pan <bo.pan@mediatek.com>
 *         YC Hung <yc.hung@mediatek.com>
 */

#ifndef __SOF_DRIVERS_AFE_MEMIF_H__
#define __SOF_DRIVERS_AFE_MEMIF_H__

#include <rtos/bit.h>
#include <sof/common.h>
#include <sof/lib/dma.h>
#include <sof/trace/trace.h>
#include <user/trace.h>
#include <stdint.h>

enum {
	MEM_DIR_CAPTURE = 0,
	MEM_DIR_PLAYBACK,
};

extern const struct dma_ops memif_ops;

#endif /* __SOF_DRIVERS_AFE_MEMIF_H__ */

