/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2021 Intel Corporation. All rights reserved.
 *
 * Author: Shriram Shastry <malladi.sastry@linux.intel.com>
 *
 */

#ifndef __SOF_MATH__SQRTLOOKUP__H
#define __SOF_MATH__SQRTLOOKUP__H

#include <stdint.h>

uint16_t sqrt_int16(uint16_t u);
#endif
