/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2021 Intel Corporation. All rights reserved.
 *
 * Author: Shriram Shastry <malladi.sastry@linux.intel.com>
 *
 */
#ifndef __SOF_MATH_POWER_H__
#define __SOF_MATH_POWER_H__

/* Include Files */
#include <stdint.h>
/* Function Declarations */
int32_t power_int32(int32_t b, int32_t e);

#endif
