/* SPDX-License-Identifier: BSD-3-Clause */
/*
 * Copyright(c) 2024 MediaTek. All rights reserved.
 *
 * Author: Andrew Perepech <andrew.perepech@mediatek.com>
 */

#ifndef _MT_8365_AFE_COMMON_H_
#define _MT_8365_AFE_COMMON_H_

/* AFE: the abbreviation for Audio Front End  */

enum {
	MT8365_MEMIF_START,
	MT8365_MEMIF_DL_START = MT8365_MEMIF_START,
	MT8365_MEMIF_DL1 = MT8365_MEMIF_DL_START,
	MT8365_MEMIF_DL2,
	MT8365_MEMIF_TDM_OUT,
	MT8365_MEMIF_DL_END,
	MT8365_MEMIF_UL_START = MT8365_MEMIF_DL_END,
	MT8365_MEMIF_AWB = MT8365_MEMIF_UL_START,
	MT8365_MEMIF_VUL,
	MT8365_MEMIF_VUL2,
	MT8365_MEMIF_VUL3,
	MT8365_MEMIF_TDM_IN,
	MT8365_MEMIF_UL_END,
	MT8365_MEMIF_END = MT8365_MEMIF_UL_END,
	MT8365_MEMIF_DL_NUM = (MT8365_MEMIF_DL_END - MT8365_MEMIF_DL_START),
	MT8365_MEMIF_UL_NUM = (MT8365_MEMIF_UL_END - MT8365_MEMIF_UL_START),
	MT8365_MEMIF_NUM = (MT8365_MEMIF_END - MT8365_MEMIF_START),
};

enum {
	MT8365_AFE_IRQ_1,
	MT8365_AFE_IRQ_2,
	MT8365_AFE_IRQ_3,
	MT8365_AFE_IRQ_4,
	MT8365_AFE_IRQ_5,
	MT8365_AFE_IRQ_6,
	MT8365_AFE_IRQ_7,
	MT8365_AFE_IRQ_8,
	MT8365_AFE_IRQ_9,
	MT8365_AFE_IRQ_10,
	MT8365_AFE_IRQ_NUM,
};

enum {
	MT8365_AFE_IO_INT_ADDA_OUT,
	MT8365_AFE_IO_2ND_I2S,
	MT8365_AFE_IO_INT_ADDA_IN,
	MT8365_AFE_IO_DMIC,
	MT8365_DAI_NUM,
};

#endif /* _MT_8365_AFE_COMMON_H_ */
