/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2024 Intel Corporation. All rights reserved.
 *
 * Author: Baofeng Tian <Baofeng.tian@intel.com>
 */

#ifndef __ZEPHYR_RTOS_STRING_MACRO_H__
#define __ZEPHYR_RTOS_STRING_MACRO_H__

#include <zephyr/toolchain/common.h>

#endif /* __ZEPHYR_RTOS_STRING_MACRO_H__ */
