// SPDX-License-Identifier: BSD-3-Clause

#include <sof/lib/pm_runtime.h>

#include <inttypes.h>

void pm_runtime_enable(enum pm_runtime_context context, uint32_t index)
{}

void pm_runtime_disable(enum pm_runtime_context context, uint32_t index)
{}
