/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2025 Intel Corporation. All rights reserved.
 *
 */

#ifndef __SOF_AUDIO_COEFFICIENTS_SRC_SRC_IPC4_INT32_DEFINE_H__
#define __SOF_AUDIO_COEFFICIENTS_SRC_SRC_IPC4_INT32_DEFINE_H__

/* SRC constants */
#define MAX_FIR_DELAY_SIZE 1305
#define MAX_OUT_DELAY_SIZE 3844
#define MAX_BLK_IN 80
#define MAX_BLK_OUT 64
#define NUM_IN_FS 16
#define NUM_OUT_FS 13
#define STAGE1_TIMES_MAX 32
#define STAGE2_TIMES_MAX 40
#define STAGE_BUF_SIZE 840
#define NUM_ALL_COEFFICIENTS 76328

#endif /* __SOF_AUDIO_COEFFICIENTS_SRC_SRC_IPC4_INT32_DEFINE_H__ */
