/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2023 Intel Corporation. All rights reserved.
 */

#ifndef __RTOS_SYMBOL_H__
#define __RTOS_SYMBOL_H__

#include <zephyr/llext/symbol.h>

#endif
