/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2025 Intel Corporation.
 *
 * Author: Adrian Bonislawski <adrian.bonislawski@intel.com>
 */

#ifndef TESTER_SIMPLE_DRAM_TEST
#define TESTER_SIMPLE_DRAM_TEST

#include <stddef.h>
#include <stdint.h>

#include "tester.h"

extern const struct tester_test_case_interface tester_interface_simple_dram_test;

#endif /* TESTER_DUMMY_TEST */
