// SPDX-License-Identifier: BSD-3-Clause
//
// Copyright(c) 2024 Intel Corporation.
//

#ifndef __ZEPHYR_RTOS_MUTEX_H__
#define __ZEPHYR_RTOS_MUTEX_H__

#include <zephyr/kernel.h> /* k_mutex_*() */

#endif /* __ZEPHYR_RTOS_MUTEX_H__ */
