/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2022 Google LLC.  All rights reserved.
 * Author: Andy Ross <andyross@google.com>
 */
#ifndef PLATFORM_POSIX_PLATFORM_DRIVERS_INTERRUPT_H
#define PLATFORM_POSIX_PLATFORM_DRIVERS_INTERRUPT_H

#define PLATFORM_IRQ_FIRST_CHILD 0

#endif /* PLATFORM_POSIX_PLATFORM_DRIVERS_INTERRUPT_H */
