/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2022 Intel Corporation. All rights reserved.
 */

#define LENGTH_RECTANGULAR 16

static const int16_t ref_rectangular[LENGTH_RECTANGULAR] = {
	32767,
	32767,
	32767,
	32767,
	32767,
	32767,
	32767,
	32767,
	32767,
	32767,
	32767,
	32767,
	32767,
	32767,
	32767,
	32767,
};
