/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2017 Intel Corporation. All rights reserved.
 *
 * Author: Keyon Jie <yang.jie@linux.intel.com>
 *         Rander Wang <rander.wang@intel.com>
 */

#ifndef __SOF_DRIVERS_HDA_DMA_H__
#define __SOF_DRIVERS_HDA_DMA_H__

#include <sof/lib/dma.h>

extern const struct dma_ops hda_host_dma_ops;
extern const struct dma_ops hda_link_dma_ops;

#endif /* __SOF_DRIVERS_HDA_DMA_H__ */
