/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2019 Intel Corporation. All rights reserved.
 *
 * Author: Tomasz Lauda <tomasz.lauda@linux.intel.com>
 */

#ifndef __SOF_LIB_SHIM_H__
#define __SOF_LIB_SHIM_H__

#include <platform/lib/shim.h>

#endif /* __SOF_LIB_SHIM_H__ */
