/* SPDX-License-Identifier: BSD-3-Clause */
/*
 * Copyright(c) 2022 MediaTek. All rights reserved.
 *
 * Author: Chunxu Li <chunxu.li@mediatek.com>
 */

#ifndef _MT_8186_AFE_COMMON_H_
#define _MT_8186_AFE_COMMON_H_

/* AFE: the abbreviation for Audio Front End  */

enum {
	MT8186_MEMIF_START,
	MT8186_MEMIF_DL_START = MT8186_MEMIF_START,
	MT8186_MEMIF_DL1 = MT8186_MEMIF_DL_START,
	MT8186_MEMIF_DL2,
	MT8186_MEMIF_DL_END,
	MT8186_MEMIF_UL_START = MT8186_MEMIF_DL_END,
	MT8186_MEMIF_UL1 = MT8186_MEMIF_UL_START,
	MT8186_MEMIF_UL2,
	MT8186_MEMIF_UL_END,
	MT8186_MEMIF_END = MT8186_MEMIF_UL_END,
	MT8186_MEMIF_DL_NUM = (MT8186_MEMIF_DL_END - MT8186_MEMIF_DL_START),
	MT8186_MEMIF_UL_NUM = (MT8186_MEMIF_UL_END - MT8186_MEMIF_UL_START),
	MT8186_MEMIF_NUM = (MT8186_MEMIF_END - MT8186_MEMIF_START),
};

enum {
	MT8186_IRQ_0,
	MT8186_IRQ_1,
	MT8186_IRQ_2,
	MT8186_IRQ_3,
	MT8186_IRQ_4,
	MT8186_IRQ_5,
	MT8186_IRQ_6,
	MT8186_IRQ_7,
	MT8186_IRQ_8,
	MT8186_IRQ_9,
	MT8186_IRQ_10,
	MT8186_IRQ_11,
	MT8186_IRQ_12,
	MT8186_IRQ_13,
	MT8186_IRQ_14,
	MT8186_IRQ_15,
	MT8186_IRQ_16,
	MT8186_IRQ_17,
	MT8186_IRQ_18,
	MT8186_IRQ_19,
	MT8186_IRQ_20,
	MT8186_IRQ_21,
	MT8186_IRQ_22,
	MT8186_IRQ_23,
	MT8186_IRQ_24,
	MT8186_IRQ_25,
	MT8186_IRQ_26,
	MT8186_IRQ_NUM,
};

enum {
	MT8186_AFE_IO_I2S1,
	MT8186_AFE_IO_I2S3,
	MT8186_AFE_IO_UL_SRC1,
	MT8186_AFE_IO_I2S0,
	MT8186_DAI_NUM,
};

#endif
