/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2021 Intel Corporation. All rights reserved.
 */
#include <tools/rimage/config/platform.toml>
#define LOAD_TYPE "2"
#include "../google_rtc_audio_processing.toml"

[module]
count = __COUNTER__
