/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2022 Google LLC.  All rights reserved.
 * Author: Andy Ross <andyross@google.com>
 */
#define PLATFORM_PRIMARY_CORE_ID 0

void platform_wait_for_interrupt(int lvl);
