/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2022 Google LLC.  All rights reserved.
 * Author: Andy Ross <andyross@google.com>
 */

#define DAI_NUM_ALH_BI_DIR_LINKS_GROUP 4
#define DAI_NUM_HDA_IN 10
#define DAI_NUM_HDA_OUT 9
#define DAI_NUM_SSP_BASE 6
#define DAI_NUM_ALH_BI_DIR_LINKS 16

