/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2019 Intel Corporation. All rights reserved.
 *
 * Author: Seppo Ingalsuo <seppo.ingalsuo@linux.intel.com>
 */

#ifndef __SOF_AUDIO_COEFFICIENTS_SRC_SRC_TINY_INT16_DEFINE_H__
#define __SOF_AUDIO_COEFFICIENTS_SRC_SRC_TINY_INT16_DEFINE_H__

/* SRC constants */
#define MAX_FIR_DELAY_SIZE 592
#define MAX_OUT_DELAY_SIZE 577
#define MAX_BLK_IN 25
#define MAX_BLK_OUT 25
#define NUM_IN_FS 7
#define NUM_OUT_FS 7
#define STAGE1_TIMES_MAX 21
#define STAGE2_TIMES_MAX 25
#define STAGE_BUF_SIZE 168
#define NUM_ALL_COEFFICIENTS 2248

#endif /* __SOF_AUDIO_COEFFICIENTS_SRC_SRC_TINY_INT16_DEFINE_H__ */
