/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2017 Intel Corporation. All rights reserved.
 *
 * Author: Liam Girdwood <liam.r.girdwood@linux.intel.com>
 */

/**
 * \file arch/xtensa/include/arch/schedule/task.h
 * \brief Arch task header file
 * \authors Liam Girdwood <liam.r.girdwood@linux.intel.com>
 */

#ifdef __POSIX_RTOS_TASK_H__

#ifndef __ARCH_SCHEDULE_TASK_H__
#define __ARCH_SCHEDULE_TASK_H__

#endif /* __ARCH_SCHEDULE_TASK_H__ */

#else

#error "This file shouldn't be included from outside of XTOS's posix/task.h"

#endif /* __POSIX_RTOS_TASK_H__ */
