// SPDX-License-Identifier: BSD-3-Clause
//
// Copyright(c) 2020 Intel Corporation. All rights reserved.
//

#ifndef __ZEPHYR_RTOS_KERNEL_H__
#define __ZEPHYR_RTOS_KERNEL_H__

#include <zephyr/kernel.h>

#endif /* __ZEPHYR_RTOS_KERNEL_H__ */
