/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2022 Intel Corporation. All rights reserved.
 */

#include <ipc/probe_dma_frame.h>

#if CONFIG_IPC_MAJOR_4
#include <ipc4/probe.h>
#else
#include <ipc3/probe.h>
#endif
