// SPDX-License-Identifier: BSD-3-Clause
/*
 * Copyright(c) 2024 MediaTek. All rights reserved.
 *
 * Author: Darren Ye <darren.ye@mediatek.com>
 */

#include <stdint.h>

#include <sof/drivers/afe-sgen.h>
#include <sof/lib/io.h>
#include <sof/lib/uuid.h>
#include <sof/trace/trace.h>
#include <mt8196-afe-reg.h>
#include <mt8196-afe-common.h>

void afe_sinegen_enable(void)
{
}

void afe_sinegen_disable(void)
{
}
