/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2017 Intel Corporation. All rights reserved.
 *
 * Author: Seppo Ingalsuo <seppo.ingalsuo@linux.intel.com>
 */

#ifndef __USER_TONE_H__
#define __USER_TONE_H__

#define SOF_TONE_IDX_FREQUENCY		0
#define SOF_TONE_IDX_AMPLITUDE		1
#define SOF_TONE_IDX_FREQ_MULT		2
#define SOF_TONE_IDX_AMPL_MULT		3
#define SOF_TONE_IDX_LENGTH		4
#define SOF_TONE_IDX_PERIOD		5
#define SOF_TONE_IDX_REPEATS		6
#define SOF_TONE_IDX_LIN_RAMP_STEP	7

#endif /* __USER_TONE_H__ */
