/* SPDX-License-Identifier: BSD-3-Clause */
/*
 * Copyright(c) 2023 MediaTek. All rights reserved.
 *
 * Author: Trevor Wu <trevor.wu@mediatek.com>
 */

#ifndef _MT_8188_AFE_COMMON_H_
#define _MT_8188_AFE_COMMON_H_

enum {
	MT8188_MEMIF_START,
	MT8188_MEMIF_DL_START = MT8188_MEMIF_START,
	MT8188_MEMIF_DL2 = MT8188_MEMIF_DL_START,
	MT8188_MEMIF_DL3,
	MT8188_MEMIF_DL_END,
	MT8188_MEMIF_UL_START = MT8188_MEMIF_DL_END,
	MT8188_MEMIF_UL4 = MT8188_MEMIF_UL_START,
	MT8188_MEMIF_UL5,
	MT8188_MEMIF_UL_END,
	MT8188_MEMIF_END = MT8188_MEMIF_UL_END,
	MT8188_MEMIF_DL_NUM = (MT8188_MEMIF_DL_END - MT8188_MEMIF_DL_START),
	MT8188_MEMIF_UL_NUM = (MT8188_MEMIF_UL_END - MT8188_MEMIF_UL_START),
	MT8188_MEMIF_NUM = (MT8188_MEMIF_END - MT8188_MEMIF_START),
};

enum {
	MT8188_IRQ_1,
	MT8188_IRQ_2,
	MT8188_IRQ_3,
	MT8188_IRQ_8,
	MT8188_IRQ_9,
	MT8188_IRQ_10,
	MT8188_IRQ_13,
	MT8188_IRQ_14,
	MT8188_IRQ_15,
	MT8188_IRQ_16,
	MT8188_IRQ_17,
	MT8188_IRQ_18,
	MT8188_IRQ_19,
	MT8188_IRQ_20,
	MT8188_IRQ_21,
	MT8188_IRQ_22,
	MT8188_IRQ_23,
	MT8188_IRQ_24,
	MT8188_IRQ_25,
	MT8188_IRQ_26,
	MT8188_IRQ_27,
	MT8188_IRQ_28,
	MT8188_IRQ_NUM,
};

enum {
	MT8188_AFE_IO_ETDM2_OUT, /* speaker */
	MT8188_AFE_IO_ETDM1_OUT, /* headset out */
	MT8188_AFE_IO_ADDA, /* DMIC */
	MT8188_AFE_IO_ETDM2_IN, /* headset mic */
	MT8188_DAI_NUM,
};

#endif
