// SPDX-License-Identifier: BSD-3-Clause
//
// Copyright(c) 2021 Xperi. All rights reserved.
//
// Author: Mark Barton <mark.barton@xperi.com>
#ifndef __SOF_AUDIO_DTS_SOF_INTERFACE_RESULT_H__
#define __SOF_AUDIO_DTS_SOF_INTERFACE_RESULT_H__


#if defined(__cplusplus)
extern "C" {
#endif

typedef enum DtsSofInterfaceResult
{
	DTS_SOF_INTERFACE_RESULT_SUCCESS = 0,

	DTS_SOF_INTERFACE_RESULT_ERROR_NO_MEMORY                            = -1001000,
	DTS_SOF_INTERFACE_RESULT_ERROR_ALREADY_INITIALIZED_MODULE           = -1001001,
	DTS_SOF_INTERFACE_RESULT_ERROR_DTS_INTERNAL_MODULE_ERROR            = -1001002,
	DTS_SOF_INTERFACE_RESULT_INVALID_SDK_VERSION                        = -1001003,
	DTS_SOF_INTERFACE_RESULT_ERROR_PARAMETER_NULL                       = -1001004,
	DTS_SOF_INTERFACE_RESULT_ERROR_INVALID_PARAMETER_SYSTEM_LAYOUT      = -1001005,
	DTS_SOF_INTERFACE_RESULT_ERROR_INVALID_PARAMETER_SYSTEM_FORMAT      = -1001006,
	DTS_SOF_INTERFACE_RESULT_ERROR_INVALID_PARAMETER_SYSTEM_SAMPLE_RATE = -1001007,
	DTS_SOF_INTERFACE_RESULT_ERROR_INCORRECT_LIFETIME_STATE             = -1001008,
	DTS_SOF_INTERFACE_RESULT_ERROR_INVALID_PARAMETER_ID                 = -1001009,
	DTS_SOF_INTERFACE_RESULT_ERROR_INVALID_CONFIGURATION_DATA           = -1001010,

} DtsSofInterfaceResult;

#if defined(__cplusplus)
}
#endif

#endif //  __SOF_AUDIO_DTS_SOF_INTERFACE_RESULT_H__
