/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2024 Intel Corporation. All rights reserved.
 *
 */

#ifndef __SOF_AUDIO_COEFFICIENTS_SRC_SRC_LITE_INT32_DEFINE_H__
#define __SOF_AUDIO_COEFFICIENTS_SRC_SRC_LITE_INT32_DEFINE_H__

/* SRC constants */
#define MAX_FIR_DELAY_SIZE 384
#define MAX_OUT_DELAY_SIZE 196
#define MAX_BLK_IN 21
#define MAX_BLK_OUT 20
#define NUM_IN_FS 3
#define NUM_OUT_FS 2
#define STAGE1_TIMES_MAX 21
#define STAGE2_TIMES_MAX 10
#define STAGE_BUF_SIZE 210
#define NUM_ALL_COEFFICIENTS 4952

#endif /* __SOF_AUDIO_COEFFICIENTS_SRC_SRC_LITE_INT32_DEFINE_H__ */
