/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2018 Intel Corporation. All rights reserved.
 *
 * Author: Slawomir Blauciak <slawomir.blauciak@linux.intel.com>
 */

#include <stdint.h>
#include <sof/audio/component.h>

#define SOF_COMP_MOCK ((uint32_t)-1)

void sys_comp_mock_init(void);
int mock_comp_register(struct comp_driver_info *info);
