/* SPDX-License-Identifier: BSD-3-Clause */
/*
 * Copyright(c) 2023 Intel Corporation. All rights reserved.
 */

#ifndef __POSIX_RTOS_INIT_H__
#define __POSIX_RTOS_INIT_H__

#define SOF_MODULE_INIT(name, init)

#endif /* __POSIX_RTOS_INIT_H__ */
