/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2025 Dolby Laboratories. All rights reserved.
 */
#include <tools/rimage/config/platform.toml>
#define LOAD_TYPE "2"
#include "../dax.toml"

[module]
count = __COUNTER__
