/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2025 Intel Corporation.
 *
 * Author: Marcin Szkudlinski <marcin.szkudlinski@linux.intel.com>
 */

#ifndef TESTER_DUMMY_TEST
#define TESTER_DUMMY_TEST

#include <stddef.h>
#include <stdint.h>

#include "tester.h"

extern const struct tester_test_case_interface tester_interface_dummy_test;

#endif /* TESTER_DUMMY_TEST */
