/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2019 Intel Corporation. All rights reserved.
 *
 * Author: Karol Trzcinski <karolx.trzcinski@linux.intel.com>
 */

/**
 * \file xtos/include/sof/compiler_info.h
 * \brief Compiler version and name descriptor
 * \author Karol Trzcinski <karolx.trzcinski@linux.intel.com>
 */

#ifndef __SOF_COMPILER_INFO_H__
#define __SOF_COMPILER_INFO_H__

#include <arch/compiler_info.h>

#endif /* __SOF_COMPILER_INFO_H__ */
