/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2020 Intel Corporation. All rights reserved.
 *
 * Author: Artur Kloniecki <arturx.kloniecki@linux.intel.com>
 */

#ifndef __IPC_FW_READY_METADATA_H__
#define __IPC_FW_READY_METADATA_H__

#include <ipc/info.h>

#endif /* __IPC_FW_READY_METADATA_H__ */
