/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2020 Intel Corporation. All rights reserved.
 *
 */

#ifndef __SOF_AUDIO_COEFFICIENTS_SRC_SRC_SMALL_INT32_DEFINE_H__
#define __SOF_AUDIO_COEFFICIENTS_SRC_SRC_SMALL_INT32_DEFINE_H__

/* SRC constants */
#define MAX_FIR_DELAY_SIZE 452
#define MAX_OUT_DELAY_SIZE 401
#define MAX_BLK_IN 21
#define MAX_BLK_OUT 21
#define NUM_IN_FS 7
#define NUM_OUT_FS 7
#define STAGE1_TIMES_MAX 21
#define STAGE2_TIMES_MAX 21
#define STAGE_BUF_SIZE 168
#define NUM_ALL_COEFFICIENTS 7536

#endif /* __SOF_AUDIO_COEFFICIENTS_SRC_SRC_SMALL_INT32_DEFINE_H__ */
