/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2021 Waves Audio Ltd. All rights reserved.
 */
#ifndef MAXX_STATUS_H
#define MAXX_STATUS_H

#include <stdint.h>

/**
 * Non-zero value represents error code type
 */
typedef int32_t MaxxStatus_t;

#endif
