/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright(c) 2016 Intel Corporation. All rights reserved.
 *
 * Author: Liam Girdwood <liam.r.girdwood@linux.intel.com>
 */

#ifdef __XTOS_RTOS_SOF_H__

#ifndef __ARCH_SOF_H__
#define __ARCH_SOF_H__

#endif /* __ARCH_SOF_H__ */

#else

#error "This file shouldn't be included from outside of XTOS's rtos/sof.h"

#endif /* __XTOS_RTOS_SOF_H__ */
