/* SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright 2025 NXP
 */

#ifndef EAP_LIB_DEFINES_H_
#define EAP_LIB_DEFINES_H_

#define LVCS_MS_COEFFS_SMALL	1
#define ALGORITHM_CS		1
#define ALGORITHM_DBE		1
#define ALGORITHM_EQNB		1
#define ALGORITHM_PR_EQNB	1
#define ALGORITHM_LM		1
#define ALGORITHM_TE		1
#define ALGORITHM_TG		1
#define ALGORITHM_AVL		1
#define ALGORITHM_PSA		1
#define ALGORITHM_VC		1
#define ALGORITHM_LIMP		1
#define ALGORITHM_LIMR		1
#define ALGORITHM_XO		1
#define NXP_PLATFORM_PROTECTION	0

#endif /* EAP_LIB_DEFINES_H_ */
