/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2024 Google LLC.  All rights reserved.
 * Author: Andy Ross <andyross@google.com>
 */
#ifndef _SOF_PLATFORM_MTK_LIB_DAI_H
#define _SOF_PLATFORM_MTK_LIB_DAI_H

/* No symbols needed from this header, but included anyway */

#endif /* _SOF_PLATFORM_MTK_LIB_DAI_H */
